"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findAliasOfType = exports.findAliasOfTypes = void 0;
/**
 * Given a list of names, filters and aliases them per the request's preferences.
 */
const findAliasOfTypes = (request, allUnionNames) => {
    // If we exclude unmatched types, remove those
    let unionNames = request.options.types.matching === undefined
        ? allUnionNames
        : filterMatchingTypeNames(allUnionNames, request.options.types.matching);
    if (unionNames.length === 0) {
        return undefined;
    }
    // Remove any duplicate names, since seemingly different flags or types might resolve to the same
    unionNames = [...new Set(unionNames)];
    // Alias the unioned names into what they'll be printed as
    // We intentionally don't remove duplicates here, as some aliases might be "TODO" or similar
    unionNames = unionNames.map((type) => exports.findAliasOfType(type, request.options.types.aliases));
    return unionNames.join(" | ");
};
exports.findAliasOfTypes = findAliasOfTypes;
/**
 * Given a type name, returns its matched alias if one is found, or the type otherwise.
 *
 * @remarks
 * It's likely this could become a performance concern for projects with many type aliases.
 * If (and only if) profiling shows this to be the case, consider adding a cache to the request object.
 */
const findAliasOfType = (type, aliases) => {
    for (const [matcher, value] of aliases) {
        if (type.match(matcher) !== null) {
            return value.replace(/\{0\}/g, type);
        }
    }
    return type;
};
exports.findAliasOfType = findAliasOfType;
const filterMatchingTypeNames = (unionNames, matching) => unionNames.filter((name) => matching.some((matcher) => name.match(matcher) !== null));
//# sourceMappingURL=findAliasOfTypes.js.map