"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractRawCodefixUnionTypes = void 0;
const ts = __importStar(require("typescript"));
const arrays_1 = require("../../shared/arrays");
const findAliasOfTypes_1 = require("../aliasing/findAliasOfTypes");
/**
 * String equivalents to print for literal node types.
 */
const keywordLiteralEquivalents = new Map([
    [ts.SyntaxKind.NullKeyword, "null"],
    [ts.SyntaxKind.StringKeyword, "string"],
    [ts.SyntaxKind.BooleanKeyword, "boolean"],
    [ts.SyntaxKind.TrueKeyword, "true"],
    [ts.SyntaxKind.FalseKeyword, "false"],
    [ts.SyntaxKind.UnknownKeyword, "unknown"],
    [ts.SyntaxKind.UndefinedKeyword, "undefined"],
    [ts.SyntaxKind.NumberKeyword, "number"],
    [ts.SyntaxKind.BigIntKeyword, "bigint"],
]);
/**
 * Nodes types known to be literals allowed in types.onlyPrimitives.
 */
const primitiveNodeKinds = new Set([
    ...keywordLiteralEquivalents.keys(),
    ts.SyntaxKind.NumericLiteral,
    ts.SyntaxKind.StringLiteral,
    ts.SyntaxKind.BigIntLiteral,
]);
/**
 * String literals to instead return undefined from for results.
 *
 * @remarks
 * It's very rare that a type is exclusively `null` or `undefined`.
 * An end result of just either of those (not in a union) is more likely a TypeScript inference bug.
 */
const invalidExtractedTypeResults = new Set(["", "null", "undefined"]);
/**
 * Given a type added by a TypeScript codefix, extrats the equivalent type to add per TypeStat type settings.
 */
const extractRawCodefixUnionTypes = (request, newText) => {
    if (!newText.startsWith(": ")) {
        return undefined;
    }
    /**
     * Converts a type node into its equivalent printed string, or `undefined` to not be printed.
     */
    const processTypeNode = (type) => {
        if (ts.isUnionTypeNode(type)) {
            return processUnionTypeNode(type);
        }
        if (request.options.types.onlyPrimitives && !primitiveNodeKinds.has(type.kind)) {
            return undefined;
        }
        const textForNode = getTextForNode(type);
        const textAlias = findAliasOfTypes_1.findAliasOfType(textForNode, request.options.types.aliases);
        if (request.options.types.matching !== undefined &&
            !request.options.types.matching.some((matcher) => textAlias.match(matcher) !== null)) {
            return undefined;
        }
        return textAlias;
    };
    const processUnionTypeNode = (type) => {
        const processedTypes = type.types.map(processTypeNode);
        const filteredProcessedTypes = processedTypes.filter(arrays_1.isNotUndefined);
        // When types.onlyPrimitives is enabled, elide all types if any are complex
        // We don't want cases like `let callbackOrUndefined: undefined = () => {};`
        return processedTypes.length === filteredProcessedTypes.length ? filteredProcessedTypes.join(" | ") : undefined;
    };
    const getTextForNode = (type) => {
        const keywordEquivalent = keywordLiteralEquivalents.get(type.kind);
        if (keywordEquivalent !== undefined) {
            return keywordEquivalent;
        }
        return type.getText(request.sourceFile);
    };
    // This creates a full TypeScript source file to accomodate for difficult-to-parse syntax such as { [i: string]: {} | [] }
    // Within that source file, a dummy type is the only statement, and its nodes are parsed to retrieve type info
    const sourceText = `type _ = ${newText.substring(": ".length)}`;
    const sourceFile = ts.createSourceFile("_.tsx", sourceText, ts.ScriptTarget.Latest);
    const typeResult = processTypeNode(sourceFile.statements[0].type);
    if (typeResult === undefined || invalidExtractedTypeResults.has(typeResult)) {
        return undefined;
    }
    return typeResult;
};
exports.extractRawCodefixUnionTypes = extractRawCodefixUnionTypes;
//# sourceMappingURL=extractRawCodefixUnionTypes.js.map