"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processCodeFixActions = void 0;
const arrays_1 = require("../../shared/arrays");
const extractRawCodefixUnionTypes_1 = require("./extractRawCodefixUnionTypes");
/**
 * Applies types settings such as aliases and onlyPrimitives to code fix actions.
 */
const processCodeFixActions = (request, codeFixActions) => {
    if (request.options.types.aliases.size === 0 && request.options.types.matching === undefined && !request.options.types.onlyPrimitives) {
        return codeFixActions;
    }
    const processCodeFixAction = (codeFixes) => {
        const changes = codeFixes.changes.map(processFileTextChanges).filter(arrays_1.isNotUndefined);
        return changes.length === 0
            ? undefined
            : Object.assign(Object.assign({}, codeFixes), { changes });
    };
    const processFileTextChanges = (fileTextChanges) => {
        const textChanges = fileTextChanges.textChanges.map(processTextChange).filter(arrays_1.isNotUndefined);
        return textChanges.length === 0
            ? undefined
            : Object.assign(Object.assign({}, fileTextChanges), { textChanges });
    };
    const processTextChange = (textChange) => {
        // Triva text changes, such as "(" and ")", don't need to be checked for codefixes
        if (!/[a-z0-9]/i.test(textChange.newText)) {
            return textChange;
        }
        const processedUnionTypes = extractRawCodefixUnionTypes_1.extractRawCodefixUnionTypes(request, textChange.newText);
        if (processedUnionTypes === undefined || processedUnionTypes.length === 0) {
            return undefined;
        }
        return Object.assign(Object.assign({}, textChange), { newText: `: ${processedUnionTypes}` });
    };
    return codeFixActions.map(processCodeFixAction).filter(arrays_1.isNotUndefined);
};
exports.processCodeFixActions = processCodeFixActions;
//# sourceMappingURL=processCodeFixActions.js.map