"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixReactPropsFromUses = void 0;
const expansionMutations_1 = require("../../../../../mutations/expansions/expansionMutations");
const collectMutationsFromNodes_1 = require("../../../../collectMutationsFromNodes");
const isReactComponentNode_1 = require("../reactFiltering/isReactComponentNode");
const getComponentPropsNode_1 = require("../getComponentPropsNode");
const getPropsUsageTypes_1 = require("./getPropsUsageTypes");
/**
 * Expands the existing props type for a component from its external JSX-style declarations.
 */
const fixReactPropsFromUses = (request) => {
    return (0, collectMutationsFromNodes_1.collectMutationsFromNodes)(request, isReactComponentNode_1.isReactComponentNode, visitReactComponentNode);
};
exports.fixReactPropsFromUses = fixReactPropsFromUses;
const visitReactComponentNode = (node, request) => {
    // Grab the node used to declare the node's props type, if it exists
    const propsNode = (0, getComponentPropsNode_1.getComponentPropsNode)(request, node);
    if (propsNode === undefined) {
        return undefined;
    }
    // Find all types of props later passed to the node
    const componentAssignedTypes = (0, getPropsUsageTypes_1.getPropsUsageTypes)(request, node, propsNode);
    if (componentAssignedTypes === undefined || componentAssignedTypes.length === 0) {
        return undefined;
    }
    return (0, expansionMutations_1.createTypeExpansionMutation)(request, propsNode, componentAssignedTypes);
};
//# sourceMappingURL=index.js.map